/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.samples.actions;

import java.text.MessageFormat;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlObject;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.ResourcesType;
import org.ogf.graap.wsag.server.monitoring.IMonitoringContext;
import org.ogf.graap.wsag.server.monitoring.IServiceTermMonitoringHandler;
import org.ogf.graap.wsag4j.types.scheduling.TimeConstraintDocument;
import org.ogf.graap.wsag4j.types.scheduling.TimeConstraintType;
import org.ogf.schemas.graap.wsAgreement.AgreementType;
import org.ogf.schemas.graap.wsAgreement.ServiceTermStateDefinition;
import org.ogf.schemas.graap.wsAgreement.ServiceTermStateType;
import org.w3c.dom.Node;

/**
 * SampleSDTMonitor
 * 
 * Once agreement instance is initialized, for each service description term and guarantee term in the
 * agreement offer, a corresponding service term state and guarantee term state with a same name is created.
 * Service term states are initially in the NOT_READY state, while guarantee term states are initially in the
 * NOT_DETERMINED state.
 * 
 * The following Monitoring Handler update the RESOURCE_STD and TIME_CONSTRAINT_SDT service term states. JSDL
 * document is appended to the RESOURCE_STD service term state and reserved host name is set. TimeConstraint
 * document is appended to the TIME_CONSTRAINT_SDT service term state.
 * 
 * @author hrasheed
 * 
 */
public class SampleSDTMonitor
    implements IServiceTermMonitoringHandler
{

    private static final Logger LOG = Logger.getLogger( SampleSDTMonitor.class );

    /**
     * {@inheritDoc}
     */
    public void monitor( IMonitoringContext context ) throws Exception
    {

        try
        {

            //
            // retrieves the Service Term State by its name
            //
            ServiceTermStateType resourcesServiceTerm = context.getServiceTermStateByName( "RESOURCE_STD" );
            ResourcesType resources = loadResourcesDefinition( resourcesServiceTerm );
            resources.set( getOfferResources( context ) );
            //
            // setting the target host name which has been reserved for a required time frame
            //
            resources.getCandidateHosts().setHostNameArray( 0, "reserved_target_host" );

            ServiceTermStateType timeServiceTerm = context.getServiceTermStateByName( "TIME_CONSTRAINT_SDT" );
            TimeConstraintType timeFrame = loadTimeConstraint( timeServiceTerm );
            timeFrame.set( getOfferTimeConstraint( context ) );

            //
            // once reservation is finished, corresponding service term state(s) can be set to COMPLETED
            //
            resourcesServiceTerm.setState( ServiceTermStateDefinition.COMPLETED );
            timeServiceTerm.setState( ServiceTermStateDefinition.COMPLETED );

        }
        catch ( Exception e )
        {
            String message =
                MessageFormat.format( "Failed to update service term state(s). Reason: {0}",
                    new Object[] { e.getMessage() } );
            LOG.error( message );
            throw new Exception( message, e );
        }
    }

    //
    // Initializes resources definition document in RESOURCE_STD service term state.
    //
    private ResourcesType loadResourcesDefinition( ServiceTermStateType state )
    {

        XmlObject[] jobDefinition =
            state.selectChildren( JobDefinitionDocument.type.getDocumentElementName() );

        if ( jobDefinition.length == 0 )
        {
            LOG.trace( "Initialize resources definition in service term state." );

            JobDefinitionDocument jobdef = JobDefinitionDocument.Factory.newInstance();
            jobdef.addNewJobDefinition().addNewJobDescription().addNewResources();

            Node imported =
                state.getDomNode().getOwnerDocument()
                     .importNode( jobdef.getJobDefinition().getDomNode(), true );
            state.getDomNode().appendChild( imported );

            jobDefinition = state.selectChildren( JobDefinitionDocument.type.getDocumentElementName() );
        }

        if ( jobDefinition.length > 1 )
        {
            String msgError =
                "Multiple resources definitions founds in service term state. "
                    + "Keeping the first, removing the others.";
            LOG.debug( msgError );

            for ( int i = 1; i < jobDefinition.length; i++ )
            {
                state.getDomNode().removeChild( jobDefinition[i].getDomNode() );
            }

            jobDefinition = state.selectChildren( JobDefinitionDocument.type.getDocumentElementName() );
        }

        JobDefinitionType jobDef = (JobDefinitionType) jobDefinition[0];

        return jobDef.getJobDescription().getResources();
    }

    //
    // Initializes time constraint document in TIME_CONSTRAINT_SDT service term state.
    //
    private TimeConstraintType loadTimeConstraint( ServiceTermStateType state )
    {

        XmlObject[] constraints = state.selectChildren( TimeConstraintDocument.type.getDocumentElementName() );

        if ( constraints.length == 0 )
        {
            LOG.trace( "Initialize time contstraint in service term state." );

            TimeConstraintDocument constraint = TimeConstraintDocument.Factory.newInstance();
            constraint.addNewTimeConstraint();

            Node imported =
                state.getDomNode().getOwnerDocument()
                     .importNode( constraint.getTimeConstraint().getDomNode(), true );
            state.getDomNode().appendChild( imported );

            constraints = state.selectChildren( TimeConstraintDocument.type.getDocumentElementName() );
        }

        if ( constraints.length > 1 )
        {
            String msgError =
                "Multiple time contstraint states founds in service term state. "
                    + "Keeping the first, removing the others.";
            LOG.debug( msgError );

            for ( int i = 1; i < constraints.length; i++ )
            {
                state.getDomNode().removeChild( constraints[i].getDomNode() );
            }

            constraints = state.selectChildren( TimeConstraintDocument.type.getDocumentElementName() );
        }

        TimeConstraintType timeConstraint = (TimeConstraintType) constraints[0];

        return timeConstraint;
    }

    //
    // retrieving agreement offer from a monitoring context,
    // and reading resource definition from the agreement offer
    //
    private ResourcesType getOfferResources( IMonitoringContext context )
    {
        AgreementType agreementOffer =
            (AgreementType) context.getProperties().get( SampleCreateAgreementAction.SAMPLE_OFFER );
        return new SampleAgreementOffer( agreementOffer ).getResourceDefinition();
    }

    //
    // retrieving agreement offer from a monitoring context,
    // and reading time constraint from the agreement offer
    //
    private TimeConstraintType getOfferTimeConstraint( IMonitoringContext context )
    {
        AgreementType agreementOffer =
            (AgreementType) context.getProperties().get( SampleCreateAgreementAction.SAMPLE_OFFER );
        return new SampleAgreementOffer( agreementOffer ).getTimeConstraint();
    }

}
