/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.samples.site;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionType;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.server.actions.AbstractCreateAgreementAction;
import org.ogf.graap.wsag.server.actions.ActionInitializationException;
import org.ogf.schemas.graap.wsAgreement.ServiceDescriptionTermType;

/**
 * This action is used as example in the WSAG4J documentation.
 * 
 * @author Oliver Waeldrich
 * 
 */
// START SNIPPET: SampleCreateAgreement
public class ExampleCreateAgreementAction extends AbstractCreateAgreementAction
{

    /**
     * The createAgreement() method is invoked when an offer was received that is based on the template
     * associated with this action. Since the structure of the template (and therefore the structure of the
     * offer) is known, we can simply access and the process offer content.
     * 
     * The structure and the content of the offer can be restricted by defining Creation Constraints in the
     * agreement template.
     * 
     * {@inheritDoc}
     */
    public Agreement createAgreement( AgreementOffer offer ) throws AgreementFactoryException
    {

        //
        // This action is configured for a template that contains one
        // service description term that describes a computational service.
        // First we get the service description term.
        //
        ServiceDescriptionTermType sdt = offer.getTerms().getAll().getServiceDescriptionTermArray( 0 );

        //
        // Now select the service description from the service description term.
        // This particular service description term contains a JSDL job definition document
        // of the type JobDefinitionType. We select the document and cast it to the
        // appropriate type.
        //
        JobDefinitionType job =
            (JobDefinitionType) sdt.selectChildren( JobDefinitionDocument.type.getDocumentElementName() )[0];

        //
        // Now we can access the properties of the job ...
        //
        job.getJobDescription().getResources().getTotalResourceCount();
        job.getJobDescription().getResources().getIndividualCPUCount();
        job.getJobDescription().getResources().getTotalPhysicalMemory();

        //
        // ... and instantiate a service.
        //
        ExampleService service = new ExampleService( job );
        service.start();

        //
        // Finally we create a new agreement instance based on the received offer.
        // The agreement properties are automatically initialized based on the offer.
        // For each service description term (guarantee term) a corresponding service
        // term state (guarantee term state is initialized.)
        //
        ExampleAgreement agreement = new ExampleAgreement( offer );
        agreement.setService( service );

        return agreement;
    }

    /**
     * This method is invoked when the action is initialized. The initialization process takes place when the
     * WSAG4J server is started.
     * 
     * {@inheritDoc}
     */
    public void initialize() throws ActionInitializationException
    {
        //
        // add implementation specific code here
        //
    }
}
// END SNIPPET: SampleCreateAgreement
